@@ -1,17 +1,23 @@
-use uuid::Uuid;
-pub use ruma::{DeviceKeyAlgorithm, RoomId, UserId, UInt, RoomAliasId,  presence::PresenceState, serde::Raw, api::client::r0::sync::sync_events::*};
-use std::collections::BTreeMap;
-use ureq::{Agent, AgentBuilder, OrAnyStatus};
-use serde::{Serialize, Deserialize};
-use serde_json::{json, Value};
-use std::time::{Duration};
 use ruma::events::*;
+pub use ruma::{
+    api::client::r0::sync::sync_events::*, identifiers::RoomName, presence::PresenceState,
+    serde::Raw, DeviceKeyAlgorithm, RoomAliasId, RoomId, UInt, UserId,
+};
+use serde::{Deserialize, Serialize};
+use serde_json::{json, Value};
+use std::collections::BTreeMap;
+use std::time::Duration;
+use ureq::{Agent, AgentBuilder, OrAnyStatus};
+use uuid::Uuid;
 const BASE: &str = "/_matrix/client/v3/";
 //const TIME_TO_IDLE: Duration = Duration::from_secs(3 * 60);
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
 pub struct StorageData {
     pub auth_token: String,
     pub room_to_aliases: BTreeMap<RoomId, Vec<RoomAliasId>>,
+    pub alias_to_room: BTreeMap<RoomAliasId, RoomId>,
+    pub joined_rooms: Vec<RoomId>,
+    pub room_to_name: BTreeMap<RoomId, Box<RoomName>>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
@@ -63,25 +69,43 @@ pub struct Client {
     host: String,
     //auth_token: String,
     //pub joined_rooms: Vec<RoomId>,
-    data: StorageData,
+    pub data: StorageData,
     //username: String,
+    room_list_updated: bool,
+    //starting: bool,
 }
 
 impl Client {
-    pub fn new(host: impl Into<String>, username: impl Into<String>, server_name: impl Into<String>) -> Self {
+    pub fn new(
+        host: impl Into<String>,
+        username: impl Into<String>,
+        server_name: impl Into<String>,
+    ) -> Self {
         Self {
             //auth_token: String::new(),
             host: host.into().trim_end_matches("/").to_string(),
-            me: UserId::try_from(format!("@{}:{}",  username.into(), server_name.into()).as_str()).unwrap(),
-            agent: AgentBuilder::new().timeout_read(Duration::from_secs(3)).build(),
+            me: UserId::try_from(format!("@{}:{}", username.into(), server_name.into()).as_str())
+                .unwrap(),
+            agent: AgentBuilder::new()
+                .timeout_read(Duration::from_secs(3))
+                .build(),
             //joined_rooms: Vec::new(),
             since: None,
             data: StorageData::default(),
             //username: username.into(),
             //server: server_name.into(),
             //last_event: EventResponse::default(),
+            room_list_updated: false,
+            //starting: true,
         }
     }
+    pub fn room_list_updated(&mut self) -> bool {
+        if self.room_list_updated {
+            self.room_list_updated = false;
+            return true;
+        }
+        false
+    }
     pub fn set_auth_token(&mut self, auth_token: String) {
         let should_req_well_known = match self.host.split(":").last().unwrap().parse::<usize>() {
             Ok(port) => {
@@ -90,19 +114,20 @@ impl Client {
                 } else {
                     true
                 }
-            },
-            Err(_) => {
-                true
             }
+            Err(_) => true,
         };
         if should_req_well_known {
-            if let Ok(z) = ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call() {
+            if let Ok(z) =
+                ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call()
+            {
                 //panic!("{}", z.into_string().unwrap());
-                
                 let resp: Value = z.into_json().unwrap();
-                let base_url: String = resp["m.homeserver"]["base_url"].as_str().unwrap().to_string();
+                let base_url: String = resp["m.homeserver"]["base_url"]
+                    .as_str()
+                    .unwrap()
+                    .to_string();
                 self.host = base_url.trim_end_matches("/").to_string();
-                
                 //self.me = UserId::try_from(format!("@{}:{}", self.me.localpart(), self.host.clone().split_once("://").unwrap().1).as_str()).unwrap();
             }
         }
@@ -116,23 +141,23 @@ impl Client {
                 } else {
                     true
                 }
-            },
-            Err(_) => {
-                true
             }
+            Err(_) => true,
         };
         if should_req_well_known {
-            if let Ok(z) = ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call() {
+            if let Ok(z) =
+                ureq::get(&format!("{}/.well-known/matrix/client", self.host.clone())).call()
+            {
                 //panic!("{}", z.into_string().unwrap());
-                
                 let resp: Value = z.into_json().unwrap();
-                let base_url: String = resp["m.homeserver"]["base_url"].as_str().unwrap().to_string();
+                let base_url: String = resp["m.homeserver"]["base_url"]
+                    .as_str()
+                    .unwrap()
+                    .to_string();
                 self.host = base_url.trim_end_matches("/").to_string();
-                
                 //self.me = UserId::try_from(format!("@{}:{}", self.me.localpart(), self.host.clone().split_once("://").unwrap().1).as_str()).unwrap();
             }
         }
-        
         let content = json!({
             "type": "m.login.password",
             "identifier": {
@@ -141,7 +166,11 @@ impl Client {
             },
             "password": password
         });
-        let val: Value = self.post_r0("login", content, None).unwrap().into_json().unwrap();
+        let val: Value = self
+            .post_r0("login", content, None)
+            .unwrap()
+            .into_json()
+            .unwrap();
         //println!("{:#}", val);
         self.data.auth_token = val["access_token"].as_str().unwrap().to_string();
         self.me = UserId::try_from(val["user_id"].as_str().unwrap()).unwrap();
@@ -149,8 +178,19 @@ impl Client {
     pub fn get_me(&self) -> UserId {
         self.me.clone()
     }
-    fn post(&self, url_ext: impl Into<String>, content: Value, query_pairs: Option<Vec<(String, String)>>) -> Result<ureq::Response, ureq::Transport> {
-        let mut r = self.agent.post(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str()).set("Authorization", format!("Bearer {}", self.data.auth_token).as_str());
+    fn post(
+        &self,
+        url_ext: impl Into<String>,
+        content: Value,
+        query_pairs: Option<Vec<(String, String)>>,
+    ) -> Result<ureq::Response, ureq::Transport> {
+        let mut r = self
+            .agent
+            .post(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str())
+            .set(
+                "Authorization",
+                format!("Bearer {}", self.data.auth_token).as_str(),
+            );
         if let Some(pairs) = query_pairs {
             for pair in pairs {
                 r = r.query(pair.0.as_str(), pair.1.as_str());
@@ -159,7 +199,6 @@ impl Client {
         //let x = r.send_json(content).or_any_status();
         //println!("{:#?}", x);
         //x
-        
         let mut resp = r.clone().send_json(content.clone()).or_any_status();
         while let Err(_e) = resp {
             resp = r.clone().send_json(content.clone()).or_any_status();
@@ -168,8 +207,27 @@ impl Client {
         resp
     }
 
-    fn post_r0(&self, url_ext: impl Into<String>, content: Value, query_pairs: Option<Vec<(String, String)>>) -> Result<ureq::Response, ureq::Transport> {
-        let mut r = self.agent.post(format!("{}{}{}", self.host, BASE.replace("v3", "r0"), url_ext.into()).as_str()).set("Authorization", format!("Bearer {}", self.data.auth_token).as_str());
+    fn post_r0(
+        &self,
+        url_ext: impl Into<String>,
+        content: Value,
+        query_pairs: Option<Vec<(String, String)>>,
+    ) -> Result<ureq::Response, ureq::Transport> {
+        let mut r = self
+            .agent
+            .post(
+                format!(
+                    "{}{}{}",
+                    self.host,
+                    BASE.replace("v3", "r0"),
+                    url_ext.into()
+                )
+                .as_str(),
+            )
+            .set(
+                "Authorization",
+                format!("Bearer {}", self.data.auth_token).as_str(),
+            );
         if let Some(pairs) = query_pairs {
             for pair in pairs {
                 r = r.query(pair.0.as_str(), pair.1.as_str());
@@ -178,7 +236,6 @@ impl Client {
         //let x = r.send_json(content).or_any_status();
         //println!("{:#?}", x);
         //x
-        
         let mut resp = r.clone().send_json(content.clone()).or_any_status();
         while let Err(_e) = resp {
             resp = r.clone().send_json(content.clone()).or_any_status();
@@ -186,8 +243,19 @@ impl Client {
         }
         resp
     }
-    fn put(&self, url_ext: impl Into<String>, content: Value, query_pairs: Option<Vec<(String, String)>>) -> Result<ureq::Response, ureq::Transport> {
-        let mut r = self.agent.put(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str()).set("Authorization", format!("Bearer {}", self.data.auth_token).as_str());
+    fn put(
+        &self,
+        url_ext: impl Into<String>,
+        content: Value,
+        query_pairs: Option<Vec<(String, String)>>,
+    ) -> Result<ureq::Response, ureq::Transport> {
+        let mut r = self
+            .agent
+            .put(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str())
+            .set(
+                "Authorization",
+                format!("Bearer {}", self.data.auth_token).as_str(),
+            );
         if let Some(pairs) = query_pairs {
             for pair in pairs {
                 r = r.query(pair.0.as_str(), pair.1.as_str());
@@ -201,20 +269,28 @@ impl Client {
         }
         resp
     }
-    fn get(&self, url_ext: impl Into<String>, query_pairs: Option<Vec<(String, String)>>) -> Result<ureq::Response, ureq::Transport> {
-        let mut r = self.agent.get(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str()).set("Authorization", format!("Bearer {}", self.data.auth_token).as_str());
+    fn get(
+        &self,
+        url_ext: impl Into<String>,
+        query_pairs: Option<Vec<(String, String)>>,
+    ) -> Result<ureq::Response, ureq::Transport> {
+        let mut r = self
+            .agent
+            .get(format!("{}{}{}", self.host, BASE, url_ext.into()).as_str())
+            .set(
+                "Authorization",
+                format!("Bearer {}", self.data.auth_token).as_str(),
+            );
         if let Some(pairs) = query_pairs {
             for pair in pairs {
                 r = r.query(pair.0.as_str(), pair.1.as_str());
             }
         }
         let mut resp = r.clone().call().or_any_status();
-        
         while let Err(_e) = resp {
             resp = r.clone().call().or_any_status();
             //eprintln!("{}", e);
         }
-        
         //println!("{:#?}", resp);
         resp
     }
@@ -224,15 +300,43 @@ impl Client {
         let rid = room_id.to_string();
         let room = urlencoding::encode(rid.as_str());
         let content = json!({});
-        self.post(format!("rooms/{}/join", room).as_str(), content, None).unwrap().into_json().unwrap()
+        self.post(format!("rooms/{}/join", room).as_str(), content, None)
+            .unwrap()
+            .into_json()
+            .unwrap()
     }
     pub fn sync(&mut self) -> Result<EventResponse, Value> {
         //let mut r = self.get("sync")
         let req: ureq::Response;
         if let Some(start) = self.since.clone() {
-            req = self.get("sync", Some(vec![("since".to_string(), start), ("full_state".to_string(), "false".to_string()), ("set_presence".to_string(), format!("{}", PresenceState::Online)),  ("timeout".to_string(), "3000".to_string())])).unwrap();
+            req = self
+                .get(
+                    "sync",
+                    Some(vec![
+                        ("since".to_string(), start),
+                        ("full_state".to_string(), "false".to_string()),
+                        (
+                            "set_presence".to_string(),
+                            format!("{}", PresenceState::Online),
+                        ),
+                        ("timeout".to_string(), "3000".to_string()),
+                    ]),
+                )
+                .unwrap();
         } else {
-            req = self.get("sync", Some(vec![("full_state".to_string(), "true".to_string()), ("set_presence".to_string(), format!("{}", PresenceState::Online)), ("timeout".to_string(), "3000".to_string())])).unwrap();
+            req = self
+                .get(
+                    "sync",
+                    Some(vec![
+                        ("full_state".to_string(), "true".to_string()),
+                        (
+                            "set_presence".to_string(),
+                            format!("{}", PresenceState::Online),
+                        ),
+                        ("timeout".to_string(), "3000".to_string()),
+                    ]),
+                )
+                .unwrap();
         }
         let val: Value = req.into_json().unwrap();
         //self.get_tx().send(format!("{:#?}", val.to_string())).unwrap();
@@ -241,22 +345,51 @@ impl Client {
         if val.clone().as_object().unwrap().contains_key("error") {
             Err(val)
         } else {
-            let x: EventResponse = serde_json::from_str::<Raw<EventResponse>>(val.to_string().as_str()).unwrap().deserialize().unwrap();
+            let x: EventResponse =
+                serde_json::from_str::<Raw<EventResponse>>(val.to_string().as_str())
+                    .unwrap()
+                    .deserialize()
+                    .unwrap();
             self.since = Some(x.clone().next_batch);
             Ok(x)
         }
     }
     pub fn join_with_alias(&mut self, alias: impl Into<String>) {
         //self.post(format!("join/{}", urlencoding::encode(alias.into())), content, None).unwrap().into_json().unwrap()
-        let room_alias = alias.into();//format!("{}", alias.into());
-        let this_room_id_val: Value = self.get(format!("directory/room/{}", urlencoding::encode(room_alias.as_str()).to_string().as_str()), None).unwrap().into_json().unwrap();
-        let room_id = ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
+        let room_alias = alias.into(); //format!("{}", alias.into());
+        let this_room_id_val: Value = self
+            .get(
+                format!(
+                    "directory/room/{}",
+                    urlencoding::encode(room_alias.as_str())
+                        .to_string()
+                        .as_str()
+                ),
+                None,
+            )
+            .unwrap()
+            .into_json()
+            .unwrap();
+        let room_id =
+            ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
         //self.data.room_to_aliases.insert(room_id.clone(), vec![room_alias.try_into().unwrap()]);
         self.join_room_id(room_id);
     }
     pub fn room_id_from_alias(&self, alias: impl Into<String>) -> RoomId {
-        let room_alias = alias.into();//format!("{}", alias.into());
-        let this_room_id_val: Value = self.get(format!("directory/room/{}", urlencoding::encode(room_alias.as_str()).to_string().as_str()), None).unwrap().into_json().unwrap();
+        let room_alias = alias.into(); //format!("{}", alias.into());
+        let this_room_id_val: Value = self
+            .get(
+                format!(
+                    "directory/room/{}",
+                    urlencoding::encode(room_alias.as_str())
+                        .to_string()
+                        .as_str()
+                ),
+                None,
+            )
+            .unwrap()
+            .into_json()
+            .unwrap();
         ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap()
     }
     /*
@@ -272,51 +405,96 @@ impl Client {
     */
     pub fn send_message(&self, room_id: RoomId, message: impl Into<String>) {
         let msg = message.into();
-        let txid = Uuid::new_v4().to_simple().encode_lower(&mut Uuid::encode_buffer()).to_string();
-        self.put(format!("rooms/{}/send/m.room.message/{}", room_id.clone(), txid).as_str(), json!({"body": msg, "msgtype": "m.text"}), None).unwrap();
+        let txid = Uuid::new_v4()
+            .to_simple()
+            .encode_lower(&mut Uuid::encode_buffer())
+            .to_string();
+        self.put(
+            format!("rooms/{}/send/m.room.message/{}", room_id.clone(), txid).as_str(),
+            json!({"body": msg, "msgtype": "m.text"}),
+            None,
+        )
+        .unwrap();
     }
 
-
-    pub fn extract_messages(&self, sync_results: EventResponse) -> Vec<MessageInfo> {
+    pub fn extract_data(&mut self, sync_results: EventResponse) -> Vec<MessageInfo> {
         let mut messages: Vec<MessageInfo> = Vec::new();
         //if let RoomsmTypeData::Joined(ref x) = event_args.room_data {
         let x = sync_results.rooms.join;
         for (room_id, joined_room) in x.iter() {
             //if event_args.ctrlc_handler.should_continue() {
-            /*
-            instructions.push(Instructions::AddRoom(room_id.clone()));
+            //instructions.push(Instructions::AddRoom(room_id.clone()));
+            if !self.data.joined_rooms.contains(room_id) {
+                self.data.joined_rooms.push(room_id.clone());
+                self.room_list_updated = true;
+            }
             for event in joined_room.state.events.clone() {
-                if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) = event.deserialize().unwrap() {
-                    let aliases = room_aliases_event.content.aliases;
-                    for alias in aliases {
-                        instructions.push(Instructions::SaveRoomAlias(room_id.clone(), alias));
+                //let ev = event.deserialize().unwrap();
+                if let Ok(ev) = event.deserialize() {
+                    if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) =
+                        ev.clone()
+                    {
+                        let aliases = room_aliases_event.content.aliases;
+                        for alias in aliases {
+                            if !self.data.alias_to_room.contains_key(&alias) {
+                                self.data
+                                    .alias_to_room
+                                    .insert(alias.clone(), room_id.clone())
+                                    .unwrap();
+                            }
+                            if !self.data.room_to_aliases.contains_key(room_id) {
+                                self.data
+                                    .room_to_aliases
+                                    .insert(room_id.clone(), vec![alias])
+                                    .unwrap();
+                            } else {
+                                if !self.data.room_to_aliases[room_id].contains(&alias) {
+                                    let mut current = self.data.room_to_aliases[room_id].clone();
+                                    current.push(alias.clone());
+                                    self.data
+                                        .room_to_aliases
+                                        .insert(room_id.clone(), current)
+                                        .unwrap();
+                                }
+                            }
+                            //instructions.push(Instructions::SaveRoomAlias(room_id.clone(), alias));
+                        }
+                    } //else if let ruma::events::AnySyncStateEvent::RoomCanonicalAlias
+                    if let ruma::events::AnySyncStateEvent::RoomName(room_name_event) = ev.clone() {
+                        if let Some(room_name) = room_name_event.content.name {
+                            self.data.room_to_name.insert(room_id.clone(), room_name);
+                        }
                     }
-                } //else if let ruma::events::AnySyncStateEvent::RoomCanonicalAlias
+                }
             }
-            */
             //if !event_args.starting {
             for message_info in joined_room.timeline.events.iter().filter_map(|m| {
-                if let ruma::events::AnySyncRoomEvent::Message(message_event) = m.deserialize_as().unwrap() {
-                    if let ruma::events::AnySyncMessageEvent::RoomMessage(room_message) = message_event {
+                if let ruma::events::AnySyncRoomEvent::Message(message_event) =
+                    m.deserialize_as().unwrap()
+                {
+                    if let ruma::events::AnySyncMessageEvent::RoomMessage(room_message) =
+                        message_event
+                    {
                         let sender = room_message.sender;
-                        
                         if sender == self.me {
                             return None;
                         }
-                        
-                        if let room::message::MessageType::Text(text_message_event_content) = room_message.content.msgtype {
+
+                        if let room::message::MessageType::Text(text_message_event_content) =
+                            room_message.content.msgtype
+                        {
                             let message = text_message_event_content.body;
                             //event_args.tx.send(format!("<{}> {}", sender, message)).unwrap();
                             //if message.starts_with(&event_args.prefix) {
-                                /*
-                                let words: Vec<String> = message.split_whitespace().map(|w| w.to_string()).collect();
-                                let args = words[1..].to_vec();
-                                let mut room_aliases: Vec<RoomAliasId> = Vec::new();
-                                if event_args.room_to_aliases.contains_key(&room_id) {
-                                    room_aliases.append(&mut event_args.room_to_aliases[&room_id].clone());
-                                }
-                                */
-                            return Some(MessageInfo{
+                            /*
+                            let words: Vec<String> = message.split_whitespace().map(|w| w.to_string()).collect();
+                            let args = words[1..].to_vec();
+                            let mut room_aliases: Vec<RoomAliasId> = Vec::new();
+                            if event_args.room_to_aliases.contains_key(&room_id) {
+                                room_aliases.append(&mut event_args.room_to_aliases[&room_id].clone());
+                            }
+                            */
+                            return Some(MessageInfo {
                                 message: message.clone(),
                                 //words: words,
                                 //args: args,
@@ -334,15 +512,13 @@ impl Client {
             }
             //}
             //} //else {
-                //instructions.push(Instructions::Quit(event_args.cleanup_on_ctrlc));
-                //return instructions;
+            //instructions.push(Instructions::Quit(event_args.cleanup_on_ctrlc));
+            //return instructions;
             //}
         }
         //}
         messages
     }
-
-
 }
 
 pub struct MessageInfo {
@@ -350,4 +526,3 @@ pub struct MessageInfo {
     pub sender: UserId,
     pub message: String,
 }
-
